SystemOrganization addCategory: #TusLibrosDotCom!

Object subclass: #Cashier
	instanceVariableNames: 'merchantProcessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibrosDotCom'!

!Cashier class methodsFor: 'errors' stamp: 'GabrielFinkelstein 6/17/2012 17:41'!
canNotCheckoutAnEmptyCartErrorMessage

	^'Can not checkout an empty cart'! !

!Cashier class methodsFor: 'errors' stamp: 'GabrielFinkelstein 6/17/2012 17:52'!
canNotCheckoutWithExpiredCardErrorMessage

	^'Can not checkout with an expired credit card'! !

!Cashier class methodsFor: 'errors' stamp: 'GabrielFinkelstein 6/17/2012 18:27'!
newWithMerchantProcessor: aMerchantProcessor

	^self new initializeWithMerchantProcessor: aMerchantProcessor.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'GabrielFinkelstein 6/18/2012 17:42'!
checkout: aCart withCreditCard: aCreditCard

	| currentMonthOfYear |
	currentMonthOfYear := MonthOfYear onMonthNumber: DateAndTime today month of: DateAndTime today year.
	
	(aCart isEmpty) ifTrue: [ self error: (self class canNotCheckoutAnEmptyCartErrorMessage ) ].

	(aCreditCard isExpiredOn: currentMonthOfYear) ifTrue: [ self error: (self class canNotCheckoutWithExpiredCardErrorMessage )].
	
	merchantProcessor debitFrom aValidCreditCard.! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'GabrielFinkelstein 6/17/2012 18:28'!
initializeWithMerchantProcessor: aMerchantProcessor

	merchantProcessor := aMerchantProcessor.! !

Object subclass: #ExceptionRaisingMerchantProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibrosDotCom'!

!ExceptionRaisingMerchantProcessor class methodsFor: 'as yet unclassified' stamp: 'GabrielFinkelstein 6/18/2012 17:46'!
errorMessage

	^'Talking to merchant processor'.! !

!ExceptionRaisingMerchantProcessor methodsFor: 'as yet unclassified' stamp: 'GabrielFinkelstein 6/18/2012 17:46'!
debitFrom: aCreditCard

	^self error: (self class errorMessage).! !

Object subclass: #ShoppingCart
	instanceVariableNames: 'catalogue contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibrosDotCom'!

!ShoppingCart class methodsFor: 'errors' stamp: 'GabrielFinkelstein 6/17/2012 17:38'!
canNotAddItemNotInTheCatalogErrorMessage

	^'Can not add items that are not in the catalog to a cart'
! !

!ShoppingCart class methodsFor: 'errors' stamp: 'GabrielFinkelstein 6/17/2012 17:39'!
canNotAddItemsWithNegativeQuantityErrorMessage

	^'Can not add items with negative quantity to a cart'! !

!ShoppingCart class methodsFor: 'as yet unclassified' stamp: 'GabrielFinkelstein 6/17/2012 18:52'!
newWithCatalogue: aCatalogue

	^self new initializeWithCatalogue: aCatalogue! !

!ShoppingCart methodsFor: 'initializing' stamp: 'GabrielFinkelstein 6/17/2012 18:54'!
addItem: anItem withQuantity: anInteger

	(catalogue includes: anItem) ifFalse: [^self error: (self class canNotAddItemNotInTheCatalogErrorMessage) ].
	
	(anInteger > 0) ifFalse: [^self error: (self class canNotAddItemsWithNegativeQuantityErrorMessage) ].
	
	contents add: anItem withOccurrences: anInteger.

! !

!ShoppingCart methodsFor: 'initializing' stamp: 'GabrielFinkelstein 6/17/2012 18:52'!
initializeWithCatalogue: aCatalogue

	contents:= Bag new.
	catalogue:= aCatalogue.! !

!ShoppingCart methodsFor: 'accesing' stamp: 'GabrielFinkelstein 6/17/2012 17:13'!
isEmpty

	^contents isEmpty.

! !

!ShoppingCart methodsFor: 'accesing' stamp: 'GabrielFinkelstein 6/12/2012 21:49'!
quantityOf: anItem

	^contents occurrencesOf: anItem.

! !

TestCase subclass: #TusLibrosDotComTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibrosDotCom'!

!TusLibrosDotComTest methodsFor: 'shopping cart object' stamp: 'GabrielFinkelstein 6/18/2012 17:40'!
aCashier
	"Por el momento creamos una cajera sin merchant processor, ya que todava no se va a usar"
	
	^Cashier newWithMerchantProcessor: nil.


! !

!TusLibrosDotComTest methodsFor: 'catalogue object' stamp: 'GabrielFinkelstein 6/17/2012 18:55'!
aCatalogue

	| catalogue |
	catalogue := Set new.
	catalogue add: '11111'; add: '22222'.
	^catalogue.


! !

!TusLibrosDotComTest methodsFor: 'shopping cart object' stamp: 'GabrielFinkelstein 6/17/2012 18:39'!
aNonEmptyCart

	^self anEmptyCart addItem: '11111' withQuantity: 1.


! !

!TusLibrosDotComTest methodsFor: 'credit card object' stamp: 'GabrielFinkelstein 6/18/2012 17:42'!
aValidCreditCard

	^CreditCard ownedBy: 'Juan' identifiedBy: 'Perez' expiresOn: (MonthOfYear januaryOf: 2013)


! !

!TusLibrosDotComTest methodsFor: 'shopping cart object' stamp: 'GabrielFinkelstein 6/17/2012 18:49'!
anEmptyCart

	^ShoppingCart newWithCatalogue: self aCatalogue.


! !

!TusLibrosDotComTest methodsFor: 'credit card object' stamp: 'GabrielFinkelstein 6/17/2012 18:35'!
anExpiredCreditCard

	^CreditCard ownedBy: 'Juan' identifiedBy: 'Perez' expiresOn: (MonthOfYear januaryOf: 2010)


! !

!TusLibrosDotComTest methodsFor: 'Tests for ShoppingCart' stamp: 'GabrielFinkelstein 6/17/2012 17:46'!
test01ANewCartShouldBeEmpty

	self assert: self anEmptyCart isEmpty.


! !

!TusLibrosDotComTest methodsFor: 'Tests for ShoppingCart' stamp: 'GabrielFinkelstein 6/17/2012 17:46'!
test02AddingItemsToCartShouldReturnACartThatIncludesTheAddedItems

	| a |
	
	a:= self anEmptyCart addItem: '11111' withQuantity: 1;
					  addItem: '22222' withQuantity: 3.

      self assert: (a quantityOf: '11111') = 1.
	self assert: (a quantityOf: '22222') = 3.


! !

!TusLibrosDotComTest methodsFor: 'Tests for ShoppingCart' stamp: 'GabrielFinkelstein 6/17/2012 17:46'!
test03ACartShouldNotContainAnItemThatWasNotAddedToIt

	| a |

	a:= self anEmptyCart addItem: '11111' withQuantity: 1.
	self assert: (a quantityOf: '22222') = 0.



! !

!TusLibrosDotComTest methodsFor: 'Tests for ShoppingCart' stamp: 'GabrielFinkelstein 6/17/2012 17:46'!
test04AddingTheSameItemMoreThanOnceToCartShouldReturnACartThatIncludesTheSumOfTheQuantitiesAddedForSuchItem

	| a |

	a:= self anEmptyCart addItem: '11111' withQuantity: 2;
                          addItem: '11111' withQuantity: 3. 
	self assert: (a quantityOf: '11111') = 5.



! !

!TusLibrosDotComTest methodsFor: 'Tests for ShoppingCart' stamp: 'GabrielFinkelstein 6/18/2012 17:36'!
test05CanNotAddAnItemNotInTheCatalogToCart

	| cart |
	cart := self anEmptyCart.
	self should: [ cart addItem: 'INVALID_ITEM' withQuantity: 2 ]
		raise: Error
		withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart canNotAddItemNotInTheCatalogErrorMessage  ].

	self assert: cart isEmpty.
! !

!TusLibrosDotComTest methodsFor: 'Tests for ShoppingCart' stamp: 'GabrielFinkelstein 6/18/2012 17:37'!
test06CanNotAddAnItemWithNegativeQuantity

	| cart |
	cart := self anEmptyCart.
	self should: [ cart addItem: '11111' withQuantity: -1 ]
		raise: Error
		withExceptionDo: [ :anException | self assert: anException messageText = ShoppingCart canNotAddItemsWithNegativeQuantityErrorMessage ].

	self assert: cart isEmpty.
! !

!TusLibrosDotComTest methodsFor: 'Tests for Cashier' stamp: 'GabrielFinkelstein 6/18/2012 17:42'!
test07CanNotCheckoutAnEmptyCart

	self
		should: [ Cashier new checkout: self anEmptyCart withCreditCard: self aValidCreditCard ]
		raise: Error
		withExceptionDo: [ :anException | self assert: anException messageText = Cashier canNotCheckoutAnEmptyCartErrorMessage ]! !

!TusLibrosDotComTest methodsFor: 'Tests for Cashier' stamp: 'GabrielFinkelstein 6/17/2012 18:47'!
test08CanNotCheckoutWithExpiredCreditCard

	self
		should: [ self aCashier checkout: self aNonEmptyCart withCreditCard: self anExpiredCreditCard]
		raise: Error
		withExceptionDo: [ :anException | self assert: anException messageText = Cashier canNotCheckoutWithExpiredCardErrorMessage ].



! !

!TusLibrosDotComTest methodsFor: 'Tests for Cashier' stamp: 'GabrielFinkelstein 6/18/2012 17:47'!
test09CashierShouldNotValidateCreditCardThroughMerchantProcessor
	"Hacemos checkout con una tarjeta vencida y con un Merchant Processor que siempre tira una excepcion,
	para asegurarnos que la cajera nunca llama al MP para validar la tarjeta"

	| cashier |
	cashier := Cashier newWithMerchantProcessor: ExceptionRaisingMerchantProcessor new.
	self
		should: [ cashier checkout: self aNonEmptyCart withCreditCard: self anExpiredCreditCard. ]
		raise: Error
		withExceptionDo: [ :anException | self assert: anException messageText ~= ExceptionRaisingMerchantProcessor errorMessage ].


! !
